#############################################################################
##
#W  test.gd             GAP character table library             Thomas Breuer
##
#H  @(#)$Id: test.gd,v 1.2 2002/04/05 10:39:50 gap Exp $
##
#Y  Copyright (C)  2002,  Lehrstuhl D fuer Mathematik,  RWTH Aachen,  Germany
##
##  This file contains the declaration of functions to test the data
##  available in the {\GAP} Character Table Library.
##
Revision.( "ctbllib/gap4/test_gd" ) :=
    "@(#)$Id: test.gd,v 1.2 2002/04/05 10:39:50 gap Exp $";


#############################################################################
##
#F  CTblLibTestMax( )
#F  CTblLibTestMax( <tblname> )
##
##  First suppose that `CTblLibTestMax' is called with one
##  argument <tblname>,
##  which is an admissible name of the form `<grpname>M<p>'.
##  Then it is checked whether a fusion from the table with name <tblname>
##  into the table with name <grpname> is stored.
##
##  If no argument is given then all admissible character table names are
##  checked with the one argument version.
##
##  In all cases, the return value is `false' if an error occurred,
##  and `true' otherwise.
##
DeclareGlobalFunction( "CTblLibTestMax" );


#############################################################################
##
#F  CTblLibTestSylowNormalizers( )
#F  CTblLibTestSylowNormalizers( <tblname> )
##
##  First suppose that `CTblLibTestSylowNormlizers' is called with one
##  argument <tblname>,
##  which is an admissible name of the form `<grpname>N<p>'.
##  Then it is checked whether the Sylow <p> subgroup of the table with name
##  <tblname> is a nontrivial normal subgroup, and whether the order of the
##  Sylow <p> subgroup coincides with that in the table for <grpname>.
##  If the Sylow <p> subgroup is cyclic then it is additionally checked
##  whether the order of the group of <tblname> equals the order of the
##  Sylow <p> normalizer.
##
##  If no argument is given then all admissible character table names are
##  checked with the one argument version.
##
##  In all cases, the return value is `false' if an error occurred,
##  and `true' otherwise.
##
DeclareGlobalFunction( "CTblLibTestSylowNormalizers" );


#############################################################################
##
#F  CTblLibTestSubgroupFusion( <sub>, <tbl> )
##
##  If no class fusion from <sub> to <tbl> is possible or if the possible 
##  class fusions contradict the stored fusion then `false' is returned.
##  If a fusion is stored and is compatible with the possible fusions, 
##  and the fusion is not unique up to table automorphisms and if the stored
##  fusion has no `text' component then `fail' is returned.
##  Otherwise the fusion record is returned.
##
DeclareGlobalFunction( "CTblLibTestSubgroupFusion" );


#############################################################################
##
#F  CTblLibTestFactorFusion( <tbl>, <fact> )
##
##  If no class fusion from <tbl> onto <fact> is possible or if the possible 
##  factor fusions contradict the stored fusion then `false' is returned.
##  If a fusion is stored and is compatible with the possible fusions, 
##  and the fusion is not unique up to table automorphisms and if the stored
##  fusion has no `text' component then `fail' is returned.
##  Otherwise the fusion record is returned.
##
DeclareGlobalFunction( "CTblLibTestFactorFusion" );


#############################################################################
##
#F  CTblLibTestFusions( )
#F  CTblLibTestFusions( <tblname> )
##
##  First suppose that `CTblLibTestFusions' is called with one
##  argument <tblname>, which is an admissible name of a character table.
##  Then it is checked whether the subgroup and factor fusions stored on the
##  table are correct.
##  (This includes the information about uniqueness of the fusion.)
##
##  If no argument is given then all standard character table names are
##  checked with the one argument version.
##
##  In all cases, the return value is `false' if an error occurred,
##  and `true' otherwise.
##
DeclareGlobalFunction( "CTblLibTestFusions" );


#############################################################################
##
#E

