#############################################################################
##
#W  test.gi             GAP character table library             Thomas Breuer
##
#H  @(#)$Id: test.gi,v 1.3 2002/04/25 16:09:20 gap Exp $
##
#Y  Copyright (C)  2002,  Lehrstuhl D fuer Mathematik,  RWTH Aachen,  Germany
##
##  This file contains the implementations of functions to test the data
##  available in the {\GAP} Character Table Library.
##
##  1. General tools for checking character tables
##  2. ...
##
Revision.( "ctbllib/gap4/test_gi" ) :=
    "@(#)$Id: test.gi,v 1.3 2002/04/25 16:09:20 gap Exp $";


#############################################################################
##
#F  ParseBackwards( <string>, <format> )
##
BindGlobal( "ParseBackwards", function( string, format )
#T Remove this as soon as `gpisotyp' is available!
    local result, pos, j, pos2;

    # Scan the string backwards.
    result:= [];
    pos:= Length( string );
    for j in Reversed( format ) do
      if IsString( j ) then
        pos2:= pos - Length( j );
        if pos2 < 0 or string{ [ pos2+1 .. pos ] } <> j then
          return fail;
        fi;
      else
        pos2:= pos;
        while 0 < pos2 and j( string[ pos2 ] ) do
          pos2:= pos2-1;
        od;
      fi;
      if j = IsDigitChar then
        Add( result, Int( string{ [ pos2+1 .. pos ] } ) );
      else
        Add( result, string{ [ pos2+1 .. pos ] } );
      fi;
      pos:= pos2;
    od;
    if 0 < pos then
      return fail;
    fi;

    return Reversed( result );
    end );


#############################################################################
##
#F  CTblLibTestMax( )
#F  CTblLibTestMax( <tblname> )
##
InstallGlobalFunction( CTblLibTestMax, function( arg )
    local result, parse, sub, tbl, fus, reps, res, name;

    # Initialize the result.
    result:= true;

    if Length( arg ) = 1 then
#T Print( arg[1], "\n" );
      name:= arg[1];
      parse:= ParseBackwards( arg[1], [ IsChar, "m", IsDigitChar ] );
      if parse <> fail and parse[3] <> 0 and ForAny( parse[1], IsAlphaChar ) then
#T and different from 'x'!
        sub:= CharacterTable( name );
        tbl:= CharacterTable( parse[1] );
        if   sub = fail then
          Print( "#I  no character table with name `", name, "'\n" );
          result:= false;
        elif tbl = fail then
          Print( "#I  no character table with name `", parse[1], "'\n" );
          result:= false;
        elif GetFusionMap( sub, tbl ) = fail then
          Print( "#I  no fusion from `", name, "' to `", parse[1],
                 "' stored\n" );
          fus:= PossibleClassFusions( sub, tbl );
          reps:= RepresentativesFusions( AutomorphismsOfTable( sub ), fus,
                     AutomorphismsOfTable( tbl ) );
          if   Length( fus ) = 0 then
            Print( "#E  no fusion from `", name, "' to `", parse[1],
                   "' is possible\n" );
            res:= fail;
          elif Length( fus ) = 1 then
            res:= LibraryFusion( Identifier( sub ),
                      rec( name:= Identifier( tbl ),
                           map:= fus[1],
                           text:= "fusion map is unique" ) );
          elif Length( reps ) = 1 then
            res:= LibraryFusion( Identifier( sub ),
                      rec( name:= Identifier( tbl ),
                           map:= fus[1],
                           text:= "fusion map is unique up to table automorphisms" ) );
          else
            Print( "#E  ambiguous fusion from `", name, "' to `",
                   parse[1], "'\n" );
            res:= fail;
          fi;
          if res <> fail then
            Print( "#I  store the following fusion:\n", res );
          fi;
          result:= false;
        fi;
      fi;
      return result;
    elif Length( arg ) = 0 then
      for name in LIBLIST.allnames do
        result:= CTblLibTestMax( name ) and result;
      od;
    fi;

    # Return the result.
    return result;
    end );


#############################################################################
##
#F  CTblLibTestSylowNormalizers( )
#F  CTblLibTestSylowNormalizers( <tblname> )
##
InstallGlobalFunction( CTblLibTestSylowNormalizers, function( arg )
    local result, name, sub, size, nsg, classes, tbl, sizetbl, orders;

    # Initialize the result.
    result:= true;

    if Length( arg ) = 1 then
      name:= ParseBackwards( arg[1], [ IsChar, "n", IsDigitChar ] );
      if name <> fail then
        sub:= CharacterTable( name );
        if sub = fail then
          Print( "#I  no character table with name `", arg[1], "'\n" );
          result:= false;
        elif not IsPrimeInt( name[3] ) then
          Print( "#E  `", name[3], "' is not a prime for `", arg[1], "'\n" );
          result:= false;
        else
          size:= First( Collected( Factors( Size( sub ) ) ),
                        pair -> pair[1] = name[3] );
          size:= size[1]^size[2];

          # Check that the Sylow `p' subgroup is normal.
          nsg:= ClassPositionsOfNormalSubgroups( sub );
          classes:= SizesConjugacyClasses( sub );
          tbl:= CharacterTable( name[1] );
          sizetbl:= First( Collected( Factors( Size( tbl ) ) ),
                           pair -> pair[1] = name[3] );
          if   ForAll( nsg, l -> Sum( classes{ l } ) <> size ) then
            Print( "#E  `", arg[1], "': Sylow ", name[3],
                   " subgroup is not normal\n" );
            result:= false;
          elif size <> sizetbl[1]^sizetbl[2] then
            Print( "#E  `", arg[1],
                   "': does not have the same Sylow subgroup as `",
                   name[1], "'\n" );
            result:= false;
          fi;

          # If the Sylow `p' subgroup is cyclic then check the order of the
          # normalizer.
          orders:= OrdersClassRepresentatives( tbl );
          if size in orders and
             Size( sub ) <> SizesCentralizers( tbl )[ Position( orders,
                size ) ] * Phi( size ) / Number( orders, x -> x = size ) then
            Print( "#E  `", arg[1], "': has not the order of the Sylow ",
                   name[3], " subgroup of `", name[1], "'\n" );
            result:= false;
          fi;
        fi;
      fi;
    elif Length( arg ) = 0 then
      for name in LIBLIST.allnames do
        result:= CTblLibTestSylowNormalizers( name ) and result;
      od;
    fi;

    # Return the result.
    return result;
    end );


#############################################################################
##
#F  CTblLibTestSubgroupFusion( <sub>, <tbl> )
##
InstallGlobalFunction( CTblLibTestSubgroupFusion, function( sub, tbl )
    local storedfus, fus, reps, record;

    storedfus:= GetFusionMap( sub, tbl );
    fus:= PossibleClassFusions( sub, tbl );

    # Do the tables and fusions fit together?
    if   IsEmpty( fus ) then
      Print( "#E  no fusion from `", Identifier( sub ), "' into `",
             Identifier( tbl ), "' possible\n" );  
      return false;
    elif storedfus <> fail and not storedfus in fus then
      Print( "#E  stored fusion from `", Identifier( sub ), "' into `",
             Identifier( tbl ), "' is wrong\n" );  
      return false;
    fi;

    # Is the fusion perhaps unique?
    if Length( fus ) = 1 then
      return rec( name := Identifier( tbl ),
                  map  := fus[1],
                  text := "fusion map is unique" );
#T check also the uniqueness claims in the stored fusion record!!
    fi;

    # Is the fusion perhaps unique up to table automorphisms?
    reps:= RepresentativesFusions( AutomorphismsOfTable( sub ), fus,
               AutomorphismsOfTable( tbl ) );
    if Length( reps ) = 1 then
      return rec( name := Identifier( tbl ),
                  map  := fus[1],
                  text := "fusion map is unique up to table autom." );
#T check also the uniqueness claims in the stored fusion record!!
    fi;

    # Is the ambiguity of the fusion perhaps mentioned in the stored fusion?
    if storedfus = fail then
      Print( "#E  ambiguous fusion from `", Identifier( sub ), "' into `",
             Identifier( tbl ), "', no map stored\n" );
      return fail;
    fi;
    record:= First( ComputedClassFusions( sub ),
                    x -> x.name = Identifier( tbl ) and x.map = storedfus );
    if not IsBound( record.text ) then
      Print( "#E  ambiguous fusion from `", Identifier( sub ), "' into `",
             Identifier( tbl ), "', no text stored\n" );
      return fail;
    elif PositionSublist( record.text, "unique" ) <> fail then
      Print( "#E  ambiguous fusion from `", Identifier( sub ), "' into `",
             Identifier( tbl ), "', with \"unique\" in text\n" );
      return fail;
    else
      return record;
    fi;
    end );


#############################################################################
##
#F  CTblLibTestFactorFusion( <tbl>, <fact> )
##
InstallGlobalFunction( CTblLibTestFactorFusion, function( tbl, fact )
    local storedfus, quot, classes, kernels, fus, ker, f, trans, map, reps,
          record;

    storedfus:= GetFusionMap( tbl, fact );
    quot:= Size( tbl ) / Size( fact );
    classes:= SizesConjugacyClasses( tbl );
    kernels:= Filtered( ClassPositionsOfNormalSubgroups( tbl ),
                        list -> Sum( classes{ list } ) = quot );

    fus:= [];
    for ker in kernels do
      f:= CharacterTableFactorGroup( tbl, ker );
      trans:= TransformingPermutationsCharacterTables( f, fact );
      if IsRecord( trans ) then
        map:= OnTuples( GetFusionMap( tbl, f ), trans.columns );
        Append( fus, Orbit( trans.group, map, OnTuples ) );
      fi;
    od;

    # Do the tables and fusions fit together?
    if IsEmpty( fus ) then
      Print( "#E  no fusion from `", Identifier( tbl ), "' onto `",
             Identifier( fact ), "' possible\n" );
      return false;
    elif storedfus <> fail and not storedfus in fus then
      Print( "#E  stored fusion from `", Identifier( tbl ), "' onto `",
             Identifier( fact ), "' is wrong\n" );  
      return false;
    fi;

    # Is the fusion perhaps unique (with given kernel)?
    if Length( fus ) = 1 then
      return rec( name := Identifier( fact ),
                  map  := fus[1],
                  text := "fusion map is unique" );
    fi;

    # Is the fusion perhaps unique up to table automorphisms (with given kernel)?
    reps:= RepresentativesFusions( Group( () ), fus,
               AutomorphismsOfTable( fact ) );
    if Length( reps ) = 1 then
      return rec( name := Identifier( fact ),
                  map  := fus[1],
                  text := "fusion map is unique up to table autom." );
    fi;

    # Is the ambiguity of the fusion perhaps mentioned in the stored fusion?
    if storedfus = fail then
      Print( "#E  ambiguous fusion from `", Identifier( tbl ), "' onto `",
             Identifier( fact ), "', no map stored\n" );
      return fail;
    fi;
    record:= First( ComputedClassFusions( tbl ),
                    x -> x.name = Identifier( fact ) and x.map = storedfus );
#T more possibilities?
    return record;
    end );


#############################################################################
##
#F  CTblLibTestFusions( )
#F  CTblLibTestFusions( <tblname> )
##
InstallGlobalFunction( CTblLibTestFusions, function( arg )
    local result, sub, record, tbl, fus, name;
    result:= true;

    if Length( arg ) = 1 then
      sub:= CharacterTable( arg[1] );
      if sub = fail then
        Print( "#I  no character table with name `", arg[1], "'\n" );
        result:= false;
      else
        # Make a shallow copy because new fusions may be entered
        # during the tests.
        for record in ShallowCopy( ComputedClassFusions( sub ) ) do
          tbl:= CharacterTable( record.name );
          if tbl = fail then
            Print( "#I  no character table with name `", record.name, "'\n" );
            result:= false;
          elif Size( sub ) <= Size( tbl ) then
            fus:= CTblLibTestSubgroupFusion( sub, tbl );
            if not IsRecord( fus ) then
              result:= false;
            fi;
          else
            fus:= CTblLibTestFactorFusion( sub, tbl );
            if not IsRecord( fus ) then
              result:= false;
            fi;
          fi;
        od;
      fi;
    elif Length( arg ) = 0 then
      for name in LIBLIST.firstnames do
        result:= CTblLibTestFusions( name ) and result;
      od;
    fi;

    # Return the result.
    return result;
    end );


#############################################################################
##
#E

